@@ -37,6 +37,17 @@ class Instances(JavaObject):
         """
         self.enforce_type(jobject, "weka.core.Instances")
         super(Instances, self).__init__(jobject)
+        self.__attribute = javabridge.make_call(self.jobject, "attribute", "(I)Lweka/core/Attribute;")
+        self.__attribute_by_name = javabridge.make_call(self.jobject, "attribute", "(Ljava/lang/String;)Lweka/core/Attribute;")
+        self.__num_attributes = javabridge.make_call(self.jobject, "numAttributes", "()I")
+        self.__num_instances = javabridge.make_call(self.jobject, "numInstances", "()I")
+        self.__get_class_index = javabridge.make_call(self.jobject, "classIndex", "()I")
+        self.__set_class_index = javabridge.make_call(self.jobject, "setClassIndex", "(I)V")
+        self.__class_attribute = javabridge.make_call(self.jobject, "classAttribute", "()Lweka/core/Attribute;")
+        self.__get_instance = javabridge.make_call(self.jobject, "instance", "(I)Lweka/core/Instance;")
+        self.__set_instance = javabridge.make_call(self.jobject, "set", "(ILweka/core/Instance;)Lweka/core/Instance;")
+        self.__append_instance = javabridge.make_call(self.jobject, "add", "(Lweka/core/Instance;)Z")
+        self.__insert_instance = javabridge.make_call(self.jobject, "add", "(ILweka/core/Instance;)V")
 
     def __iter__(self):
         """
@@ -62,7 +73,7 @@ class Instances(JavaObject):
         :return: the number of attributes
         :rtype: int
         """
-        return javabridge.call(self.jobject, "numAttributes", "()I")
+        return self.__num_attributes()
 
     def attributes(self):
         """
@@ -78,7 +89,7 @@ class Instances(JavaObject):
         :return: the attribute
         :rtype: Attribute
         """
-        return Attribute(javabridge.call(self.jobject, "attribute", "(I)Lweka/core/Attribute;", index))
+        return Attribute(self.__attribute(index))
 
     def attribute_by_name(self, name):
         """
@@ -88,7 +99,7 @@ class Instances(JavaObject):
         :return: the attribute or None
         :rtype: Attribute
         """
-        att = javabridge.call(self.jobject, "attribute", "(Ljava/lang/String;)Lweka/core/Attribute;", name)
+        att = self.__attribute_by_name(javabridge.get_env().new_string(name))
         if att is None:
             return None
         else:
@@ -123,8 +134,8 @@ class Instances(JavaObject):
         :return: the number of instances
         :rtype: int
         """
-        return javabridge.call(self.jobject, "numInstances", "()I")
-        
+        return self.__num_instances()
+
     @property
     def class_attribute(self):
         """
@@ -132,7 +143,7 @@ class Instances(JavaObject):
         :return: the class attribute
         :rtype: Attribute
         """
-        return Attribute(javabridge.call(self.jobject, "classAttribute", "()Lweka/core/Attribute;"))
+        return Attribute(self.__class_attribute())
 
     @property
     def class_index(self):
@@ -141,7 +152,7 @@ class Instances(JavaObject):
         :return: the class index, -1 if not set
         :rtype: int
         """
-        return javabridge.call(self.jobject, "classIndex", "()I")
+        return self.__get_class_index()
 
     @class_index.setter
     def class_index(self, index):
@@ -150,7 +161,21 @@ class Instances(JavaObject):
         :param index: the new index, use -1 to unset
         :type index: int
         """
-        javabridge.call(self.jobject, "setClassIndex", "(I)V", index)
+        self.__set_class_index(index)
+
+    def has_class(self):
+        """
+        Returns whether a class attribute is set(convenience method).
+        :return: whether a class attribute is currently set
+        :rtype: bool
+        """
+        return self.class_index != -1
+
+    def no_class(self):
+        """
+        Unsets the class attribute (convenience method).
+        """
+        self.class_index = -1
 
     def class_is_first(self):
         """
@@ -172,8 +197,8 @@ class Instances(JavaObject):
         :return: the instance
         :rtype: Instance
         """
-        return Instance(javabridge.call(self.jobject, "instance", "(I)Lweka/core/Instance;", index))
-        
+        return Instance(self.__get_instance(index))
+
     def add_instance(self, inst, index=None):
         """
         Adds the specified instance to the dataset.
@@ -183,9 +208,9 @@ class Instances(JavaObject):
         :type index: int
         """
         if index is None:
-            javabridge.call(self.jobject, "add", "(Lweka/core/Instance;)Z", inst.jobject)
+            self.__append_instance(inst.jobject)
         else:
-            javabridge.call(self.jobject, "add", "(ILweka/core/Instance;)V", index, inst.jobject)
+            self.__insert_instance(index, inst.jobject)
 
     def set_instance(self, index, inst):
         """
@@ -198,7 +223,7 @@ class Instances(JavaObject):
         :rtype: Instance
         """
         return Instance(
-            javabridge.call(self.jobject, "set", "(ILweka/core/Instance;)Lweka/core/Instance;", index, inst.jobject))
+            self.__set_instance(index, inst.jobject))
             
     def delete(self, index=None):
         """
@@ -444,6 +469,14 @@ class Instance(JavaObject):
         """
         self.enforce_type(jobject, "weka.core.Instance")
         super(Instance, self).__init__(jobject)
+        self.__set_value = javabridge.make_call(self.jobject, "setValue", "(ID)V")
+        self.__get_value = javabridge.make_call(self.jobject, "value", "(I)D")
+        self.__set_string_value = javabridge.make_call(self.jobject, "setValue", "(ILjava/lang/String;)V")
+        self.__get_string_value = javabridge.make_call(self.jobject, "stringValue", "(I)Ljava/lang/String;")
+        self.__set_weight = javabridge.make_call(self.jobject, "setWeight", "(D)V")
+        self.__get_weight = javabridge.make_call(self.jobject, "weight", "()D")
+        self.__is_missing = javabridge.make_call(self.jobject, "isMissing", "(I)Z")
+        self.__class_index = javabridge.make_call(self.jobject, "classIndex", "()I")
 
     def __iter__(self):
         """
@@ -509,7 +542,7 @@ class Instance(JavaObject):
         :return: the class index, -1 if not set
         :rtype: int
         """
-        return javabridge.call(self.jobject, "classIndex", "()I")
+        return self.__class_index()
 
     def set_value(self, index, value):
         """
@@ -519,7 +552,7 @@ class Instance(JavaObject):
         :param value: the internal float value to set
         :type value: float
         """
-        javabridge.call(self.jobject, "setValue", "(ID)V", index, value)
+        self.__set_value(index, value)
 
     def get_value(self, index):
         """
@@ -529,7 +562,7 @@ class Instance(JavaObject):
         :return: the internal value
         :rtype: float
         """
-        return javabridge.call(self.jobject, "value", "(I)D", index)
+        return self.__get_value(index)
 
     def set_string_value(self, index, s):
         """
@@ -539,7 +572,7 @@ class Instance(JavaObject):
         :param s: the string value
         :type s: str
         """
-        return javabridge.call(self.jobject, "setValue", "(ILjava/lang/String;)V", index, s)
+        return self.__set_string_value(index, javabridge.get_env().new_string(s))
 
     def get_string_value(self, index):
         """
@@ -549,7 +582,7 @@ class Instance(JavaObject):
         :return: the string value
         :rtype: str
         """
-        return javabridge.call(self.jobject, "stringValue", "(I)Ljava/lang/String;", index)
+        return javabridge.get_env().get_string(self.__get_string_value(index))
 
     def get_relational_value(self, index):
         """
@@ -577,7 +610,7 @@ class Instance(JavaObject):
         :return: whether the value is missing
         :rtype: bool
         """
-        return javabridge.call(self.jobject, "isMissing", "(I)Z", index)
+        return self.__is_missing(index)
 
     def has_missing(self):
         """
@@ -594,7 +627,7 @@ class Instance(JavaObject):
         :return: the weight
         :rtype: float
         """
-        return javabridge.call(self.jobject, "weight", "()D")
+        return self.__get_weight()
 
     @weight.setter
     def weight(self, weight):
@@ -603,7 +636,7 @@ class Instance(JavaObject):
         :param weight: the weight to set
         :type weight: float
         """
-        javabridge.call(self.jobject, "setWeight", "(D)V", weight)
+        self.__set_weight(weight)
 
     @property
     def values(self):
